using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Xml;
using System.Runtime.Serialization.Formatters.Binary;
using RCRequisites;

namespace GLCDView
{
	[Serializable]
	public struct CharDataRow
	{
		public byte[] Data;
		public string Comment;
	}

	[Serializable]
	public class CharData
	{
		public string Comment;
		public byte Char;
		public int Height;
		public int Width;
		public CharDataRow[] DataRows;
	}

	[Serializable]
	public class GLCDFont
	{
		public static string DefaultChars = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.,;:-_+*#?!^/()&%$@|<>";
		public static string DefaultReference = "_g9";

		public int Size = 0;
		public Font Font;
		public string Chars = GLCDFont.DefaultChars;
		public int PaddingLeft = 1, PaddingRight = 1, PaddingTop = 1, PaddingBottom = 1;
		public string Reference = GLCDFont.DefaultReference;
		public List<CharData> CharData = new List<CharData>();
		public string Name;
		public bool FixedSize = false;

		public int glyphHeight;
		public int glyphWidth;
		public byte[] char_table;
		public int[] offset_table;
		public byte[] mapping_table;
		public byte[] width_table;

		public override string ToString()
		{
			return this.Name;
		}

		public string ExportSourceFile(string pathName)
		{
			StringBuilder trace = new StringBuilder();
			List<byte> byteArray = new List<byte>();
			
			trace.Append("#include <stdlib.h>\r\n");
			trace.Append("#include <avr/pgmspace.h>\r\n");
			trace.Append("#include \"font.h\"\r\n\r\n");
			trace.AppendFormat("const unsigned char {0}_char_table[] PROGMEM = {{\r\n", this.Name);
			this.Size = 0;
			foreach (CharData c in this.CharData)
			{
				trace.Append("// ");
				trace.Append(c.Comment);
				trace.Append("\r\n");
				for (int i = 0; i < c.DataRows.Length; i++)
				{
					for (int j = 0; j < c.DataRows[i].Data.Length; j++)
					{
						byteArray.Add(c.DataRows[i].Data[j]);
						trace.AppendFormat("0x{0:x2}, ", c.DataRows[i].Data[j]);
					}
					trace.AppendFormat("// {0} \r\n", c.DataRows[i].Comment);
				}
				trace.Append("\r\n\r\n");
			}
			this.char_table = byteArray.ToArray();
			this.Size += this.char_table.Length;
			trace.Append("};\r\n\r\n");


			trace.AppendFormat("const unsigned char {0}_mapping_table[] PROGMEM = {{\r\n", this.Name);

			byteArray.Clear();
			byte[] bytes = new byte[1];
			for (int i = 0; i <= 255; i++)
			{
				bytes[0] = (byte)i;

				int mapped = 0;
				for (int j = 0; j < this.CharData.Count; j++)
				{
					if (this.CharData[j].Char == bytes[0])
					{
						mapped = j;
						break;
					}
				}
				byteArray.Add((byte)mapped);

				string s = "0";
				if (i == 0 || (i>0 && i<32) ||
					i == 92 ||
					i == 127 ||
					i == 129 ||
					i == 141 ||
					i == 143 ||
					i == 144 ||
					i == 157)
					s = " ";
				else
					s = Encoding.Default.GetString(bytes);
				trace.AppendFormat("{2}{3} // {0} = {1}\r\n", i, s, mapped, i<255 ? ",":"");
			}
			this.mapping_table = byteArray.ToArray();
			this.Size += this.mapping_table.Length;
			trace.Append("};\r\n\r\n");

			trace.AppendFormat("const unsigned char {0}_width_table[] PROGMEM = {{\r\n", this.Name);
			byteArray.Clear();
			for (int j = 0; j < this.CharData.Count; j++)
			{
				bytes[0] = this.CharData[j].Char;
				byteArray.Add((byte)this.CharData[j].Width);
				trace.AppendFormat("{0}{2} // {1} \r\n", this.CharData[j].Width, Encoding.Default.GetString(bytes), j < this.CharData.Count-1 ? ",":"");
			}
			this.width_table = byteArray.ToArray();
			this.Size += this.width_table.Length;
			trace.Append("};\r\n\r\n");


			trace.AppendFormat("const unsigned int {0}_offset_table[] PROGMEM = {{\r\n", this.Name);
			this.offset_table = new int[this.CharData.Count];
			int offset = 0;
			for (int j = 0; j < this.CharData.Count; j++)
			{
				bytes[0] = this.CharData[j].Char;
				this.offset_table[j] = offset;
				trace.AppendFormat("{0}{2} // {1} \r\n", offset, Encoding.Default.GetString(bytes), j < this.CharData.Count-1 ? ",":"");

				offset += this.CharData[j].DataRows.Length * this.CharData[j].DataRows[0].Data.Length;
			}
			this.Size += this.offset_table.Length * 2;
			trace.Append("};\r\n\r\n");

			trace.AppendFormat("const struct FONT_DEF {0} = {{\r\n   0, 	// glyph_width\r\n   {1}, 	// glyph_height\r\n   {0}_char_table, 	// ptr to glyph_table\r\n   {0}_mapping_table, 	// ptr to mapping_table\r\n   {0}_width_table,\r\n   {0}_offset_table\r\n}};\r\n",
				this.Name, this.glyphHeight);

			trace.Append("\r\n\r\n");
			trace.AppendFormat(@"// Font: {0}
// {1}
// needed: {2} Bytes
", this.Font.ToString(), this.Chars, this.Size);

			if (!string.IsNullOrEmpty(pathName))
			{
				using (StreamWriter sw = new StreamWriter(pathName, false, Encoding.Default))
				{
					sw.Write(trace.ToString());
				}
			}
			return trace.ToString();
		}

		public string ToData()
		{
			string data = ObjectSerialize.ObjectToString(this);
			return data;
		}

		public static GLCDFont FromData(string data)
		{
			GLCDFont ret = ObjectSerialize.StringToObject(data) as GLCDFont;
			return ret;
		}
	}

}
