using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace GLCDView
{
	public partial class GLCDView : UserControl
	{
		#region Fields
		int xDimension = 240;
		int yDimension = 128;
		int dotsPerPixel = 2;
		int gridWidth = 1;
		Color gridColor = Color.WhiteSmoke;
		bool[,] gfxData;
		#endregion

		#region Events
		public event EventHandler<PixelChangedArgs> PixelChanged;
		#endregion

		#region Properties
		public int XDimension
		{
			get { return this.xDimension; }
			set
			{
				if (value > 800)
					value = 800;
				if (value < 10)
					value = 10;
				this.xDimension = value;
				ClearGfx();
				RefreshSize();
			}
		}

		public int YDimension
		{
			get { return this.yDimension; }
			set
			{
				if (value > 600)
					value = 600;
				if (value < 10)
					value = 10;
				this.yDimension = value;
				ClearGfx();
				RefreshSize();
			}
		}

		public Color GridColor
		{
			get { return this.gridColor; }
			set
			{
				this.gridColor = value;
				Invalidate();
			}
		}

		public int DotsPerPixel
		{
			get { return this.dotsPerPixel; }
			set
			{
				this.dotsPerPixel = value;
				RefreshSize();
				Invalidate();
			}
		}

		public bool ShowGrid
		{
			get { return 0 != this.gridWidth; }
			set
			{
				if (Convert.ToBoolean(this.gridWidth) != value)
				{
					this.gridWidth = value ? 1 : 0;
					RefreshSize();
					Invalidate();
				}
			}
		}
		#endregion

		public GLCDView()
		{
			InitializeComponent();

			SetStyle(System.Windows.Forms.ControlStyles.UserPaint |
				System.Windows.Forms.ControlStyles.AllPaintingInWmPaint |
				System.Windows.Forms.ControlStyles.OptimizedDoubleBuffer |
				System.Windows.Forms.ControlStyles.ResizeRedraw, true);

			RefreshSize();
			ClearGfx();
		}

		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);
			RefreshSize();
		}

		protected override void OnPaintBackground(PaintEventArgs e)
		{
			// base.OnPaintBackground(e);
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Rectangle rcBackground = new Rectangle(0, 0, this.Width, this.Height);

			using (SolidBrush brBackground = new SolidBrush(this.BackColor))
			{
				e.Graphics.FillRectangle(brBackground, rcBackground);
			}


			if (this.gridWidth > 0)
			{
				using (Pen penGrid = new Pen(this.gridColor))
				{
					//e.Graphics.DrawRectangle(penGrid, 0, 0, this.Width - 1, this.Height - 1);

					for (int x = 0; x < this.Width; x += this.dotsPerPixel + this.gridWidth)
						e.Graphics.DrawLine(penGrid, x, 0, x, this.Height - 1);

					for (int y = 0; y < this.Height; y += this.dotsPerPixel + this.gridWidth)
						e.Graphics.DrawLine(penGrid, 0, y, this.Width - 1, y);
				}
			}


			using (SolidBrush brPixel = new SolidBrush(this.ForeColor))
			{
				for (int pY = 0; pY < this.yDimension; pY++)
				{
					for (int pX = 0; pX < this.xDimension; pX++)
					{
						if (true == this.gfxData[pY, pX])
							e.Graphics.FillRectangle(brPixel, this.gridWidth + pX * (this.dotsPerPixel + this.gridWidth), this.gridWidth + pY * (this.dotsPerPixel + this.gridWidth), this.dotsPerPixel, this.dotsPerPixel);
					}
				}
			}
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			base.OnMouseMove(e);

			PixelChangedArgs arg = new PixelChangedArgs();
			arg.XPixel = e.X / (this.dotsPerPixel + this.gridWidth);
			arg.YPixel = e.Y / (this.dotsPerPixel + this.gridWidth);

			if (arg.XPixel >= this.xDimension)
				arg.XPixel = this.xDimension - 1;
			if (arg.YPixel >= this.yDimension)
				arg.YPixel = this.yDimension - 1;

			if (null != this.PixelChanged)
				this.PixelChanged(this, arg);
		}

		private void RefreshSize()
		{
			int width = this.gridWidth + this.dotsPerPixel * this.xDimension + this.xDimension * this.gridWidth;
			int height = this.gridWidth + this.dotsPerPixel * this.yDimension + this.yDimension * this.gridWidth;
			this.Size = new Size(width, height);
		}

		public void SetPixel(int x, int y, bool set)
		{
			if (x < 0 || x >= this.xDimension)
				return;
			if (y < 0 || y >= this.yDimension)
				return;
			this.gfxData[y, x] = set;
		}

		public void DrawLine(int x1, int y1, int x2, int y2, bool show)
		{
			int dy;
			int dx;
			int stepx, stepy, fraction;
			dy = y2 - y1;
			dx = x2 - x1;
			if (dy < 0)
			{
				dy = -dy;
				stepy = -1;
			}
			else
			{
				stepy = 1;
			}
			if (dx < 0)
			{
				dx = -dx;
				stepx = -1;
			}
			else
			{
				stepx = 1;
			}
			dy <<= 1;
			dx <<= 1;
			SetPixel(x1, y1, show);
			if (dx > dy)
			{
				fraction = dy - (dx >> 1);
				while (x1 != x2)
				{
					if (fraction >= 0)
					{
						y1 += stepy;
						fraction -= dx;
					}
					x1 += stepx;
					fraction += dy;
					SetPixel(x1, y1, show);
				}
			}
			else
			{
				fraction = dx - (dy >> 1);
				while (y1 != y2)
				{
					if (fraction >= 0)
					{
						x1 += stepx;
						fraction -= dy;
					}
					y1 += stepy;
					fraction += dx;
					SetPixel(x1, y1, show);
				}
			}
			this.Invalidate();
		}
		
		public void DrawRectangle(int x1, int y1, int x2, int y2, bool show)
		{
			DrawLine(x1, y1, x2, y1, show);  // up
			DrawLine(x1, y2, x2, y2, show);  // down
			DrawLine(x2, y1, x2, y2, show);  // right
			DrawLine(x1, y1, x1, y2, show);  // left
		}

		public void FillRectangle(int x1, int y1, int x2, int y2, bool show)
		{
			int i;
			for (i = y1; i <= y2; i++)
				DrawLine(x1, i, x2, i, show);
		}

		public void ClearGfx()
		{
			this.gfxData = new bool[this.yDimension, this.xDimension];



			Invalidate();
		}
	}

	public class PixelChangedArgs : EventArgs
	{
		public int XPixel, YPixel;
	}
}