#region    using        
using System;
using System.Xml;
#endregion

namespace GLCDView
{
	/// <summary>
	/// SimpleXmlDocument class provides a simpley way to generate XmlNodes and XMLAttributes.
	/// </summary>
	public class SimpleXmlDocument : XmlDocument
	{
		public SimpleXmlDocument() : base()
		{
		}
		
		/// <summary>
		/// Initializes a new instance of SimpleXmlDocument class.
		/// </summary>
		/// <param name="root">Name of the root Node for this document.</param>
		/// <param name="xsl">Name of the XSL Transform stylesheet if needed, otherwise string.Empty</param>
		public SimpleXmlDocument(string root, string xsl) : base()
		{
			this.PreserveWhitespace = false;
			this.LoadXml(root);
			if(null != xsl && string.Empty != xsl)
			{
				this.InsertBefore(this.CreateProcessingInstruction("xml-stylesheet", "type='text/xsl' href='"+xsl+"'"), this.DocumentElement);
			}
		}

		/// <summary>
		/// Creates a new XmlNode with the given name.
		/// </summary>
		/// <param name="name">Name of the XmlNode.</param>
		/// <returns>A new XmlNode.</returns>
		public XmlNode CreateXmlNode(string name)
		{
			return CreateXmlNode(null, name, string.Empty);
		}

		/// <summary>
		/// Creates a new XmlNode with the given name and inner Text.
		/// </summary>
		/// <param name="name">Name of the XmlNode.</param>
		/// <param name="innerText">innerText of the XmlNode.</param>
		/// <returns>A new XmlNode.</returns>
		public XmlNode CreateXmlNode(string name, string innerText)
		{
			return CreateXmlNode(null, name, innerText);
		}

		/// <summary>
		/// Creates a new XmlNode with the given name and inner Text for a specified parent Node.
		/// </summary>
		/// <param name="parentNode">the parent node for the new xml node.</param>
		/// <param name="name">Name of the XmlNode</param>
		/// <param name="innerText">innerText of the XmlNode</param>
		/// <returns>A new XmlNode</returns>
		public XmlNode CreateXmlNode(XmlNode parentNode, string name, string innerText)
		{
			XmlNode node = this.CreateElement(name);
			node.InnerText = innerText;
			if(null != parentNode)
				parentNode.AppendChild(node);
			return node;
		}

		/// <summary>
		/// Creates a new XmlAttribute with the given name and innerText.
		/// </summary>
		/// <param name="name">The Name of the new XmlAttribute.</param>
		/// <param name="innerText">The innerText for the new AxmAttribute</param>
		/// <returns>A new XmlAttribute</returns>
		public XmlAttribute CreateXmlAttribute(string name, string innerText)
		{
			return CreateXmlAttribute(null, name, innerText);
		}

		/// <summary>
		/// Creates a new XmlNAttribute with the given name and innerText for the given XmlNode.
		/// </summary>
		/// <param name="node">The XmlNode for this new Attribute.</param>
		/// <param name="name">The name of the Attribute.</param>
		/// <param name="innerText">The innerText for this Attribute.</param>
		/// <returns>a new XmlAttribute.</returns>
		public XmlAttribute CreateXmlAttribute(XmlNode node, string name, string innerText)
		{
			XmlAttribute attribute = this.CreateAttribute(name);
			attribute.InnerText = innerText;
			if(null != node)
				((XmlElement)node).SetAttributeNode(attribute);
			return attribute;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="node"></param>
		/// <param name="xPath"></param>
		/// <param name="defaultValue"></param>
		/// <returns></returns>
		public static string GetNodeValue(XmlNode node, string xPath, string defaultValue)
		{
			XmlNode xPathNode = node.SelectSingleNode(xPath);
			if(null != xPathNode && xPathNode.InnerText.Trim().Length > 0)
				return xPathNode.InnerText.Trim();
			return defaultValue;
		}

		public static int GetNodeValue(XmlNode node, string xPath, int defaultValue)
		{
			XmlNode xPathNode = node.SelectSingleNode(xPath);
			if (null != xPathNode && xPathNode.InnerText.Length > 0)
			{
				int ret = defaultValue;
				int.TryParse(xPathNode.InnerText, out ret);
				return ret;
			}
			return defaultValue;
		}


		public static bool GetAttributeValue(XmlAttributeCollection attributes, string attributeName, bool defaultValue)
		{
			if (null == attributes)
				return defaultValue;
			XmlAttribute att = attributes[attributeName];
			if (null == att)
				return defaultValue;
			bool ret = defaultValue;
			Boolean.TryParse(att.InnerText, out ret);
			return ret;
		}

		public static int GetAttributeValue(XmlAttributeCollection attributes, string attributeName, int defaultValue)
		{
			if (null == attributes)
				return defaultValue;
			XmlAttribute att = attributes[attributeName];
			if (null == att)
				return defaultValue;
			int ret = defaultValue;
			int.TryParse(att.InnerText, out ret);
			return ret;
		}
	}
}
